clc;
clear all;
hold on

%Reads the constant for the pump characteristics from Excel
a=xlsread('Frekvenskalkulasjon.xlsx','H14:H18');
b=xlsread('Frekvenskalkulasjon.xlsx','I14:I18');
c=xlsread('Frekvenskalkulasjon.xlsx','J14:J18');
H0=xlsread('Frekvenskalkulasjon.xlsx','K14:K18');
Qstar=xlsread('Frekvenskalkulasjon.xlsx','L14:L18');

%Constants and dimensions  
g=9.81;
rho=1000;
D=0.34;
A=(D/2)^2*pi;
L1=8;       %Length between accumulator and valve, approximately 8 meters
L2=2;       %Length between pump and accumulator, approximately 2 meters
kappa=1.4;
L_acc=0.6;
A_acc=0.0201;
%As_test=0.0001;

%The statical head. It is zero in the experiment. Can be changed to see what effect the static head has on the stability
Hst=0;

%Decides the maximum time and the time steps 
tmax=1;
deltat=0.0001;

%Read values for pressure and volume exponent from Excel
%B:50, C:100, D:150, E:200, F:250, G:300, H:350, I:400, J:500
P1=(xlsread('Frekvenskalkulasjon.xlsx','B21:B25')-1.01300)*10^5*10;
volumeexp=ones(5,1)*0.198;
p0=xlsread('Frekvenskalkulasjon.xlsx','B14:B18')-101300;

%Calculates the equivalent area and the frequencies of the mass oscillations
helmholtz=zeros(1,length(P1));
As=helmholtz;
Va=L_acc*A_acc*volumeexp;
for i=1:length(Va)
    helmholtz(i)=sqrt(A*kappa/(rho*L1)*P1(i)/Va(i))/(2*pi());
    As(i)=1/(1/A_acc+P1(i)*kappa/(g*rho*Va(i)));
    f_torb(i)=sqrt(g*A/(As(i)*L1))/(2*pi());
end

%The pump characteristics are plotted so it is easier to decide which operating point to study
x=(0:0.1:1000);
Q_plot= repmat(x,length(H0),1);
Hp_plot=zeros(length(H0),length(Q_plot));
 for i=1:length(Q_plot)
    for j=1:length(H0)
         Hp_plot(j,i)=H0(j)-a(j)*Q_plot(j,i)-b(j)*(Q_plot(j,i))^2-c(j)*(Q_plot(j,i)-Qstar(j))^2;
     end
 end

%The plot of the pump characteristics
figure(1)
plot(Q_plot',Hp_plot')
xlabel('Volume flow Q [m^3/h]')
ylabel('Head Hp [m]')
legend('25 Hz','30 Hz','35 Hz','40 Hz','45 Hz') 

%The operational point is chosen through the command window
Q0=input('What Q0 [m^3/h] do you want to study?  ');

%Here is the start pressure decided. This is found with the help of the pump characteristics and the operational point. 
Hp_Q0=zeros(5,1);
for i=1:5
    Hp_Q0(i)=H0(i)-a(i)*Q0-b(i)*Q0^2-c(i)*(Qstar(i)-Q0).^2;
end

%The slope of the pump characteristic and the slope of the system characteristic is calculated next. This is done by the help of differentiation of the equations of the characteristics. k is the loss coefficient
Pp=(-a-2*Q0*b-2*Q0*c+2*diag(c)*Qstar)*3600;    %[1/ms]    
k=(Hp_Q0-Hst)/Q0^2;
Pt=(2*k*Q0)*3600;                              %[m^3/s^2]

%Figure 2 plots the characteristics and the operational point
figure(2)
plot(Q_plot',Hp_plot',Q0,Hst+k*Q0^2,'ko')
xlabel('Volume flow Q [m^3/h]')
ylabel('Head Hp [m]')
legend('25 Hz','30 Hz','35 Hz','40 Hz','45 Hz','Operational point','Location','SouthWest')

%Greitzer's dynamic stability criteria is calculated next. If posisstable is positive: stable according to Greizer's criteria
C=zeros(5,1);
for i=1:5
    C(i)=(kappa*p0(i)/Va(i))^(-1);
end

posisstable=zeros(5,1);
for i=1:5
    posisstable(i)=Pp(i)*A*g^2/L2*C(i)*rho-1/Pt(i);
end
posisstable;


%The time vector is given here. The first column is zero, the next is deltat, the third is 2 times deltat,.. The last column is tmax-deltat
nsteg=tmax/deltat;
t=0:deltat:tmax-deltat; 

%Different matrices are defined here. They first consists of only zeroes except the first row which consists of the starting values for the different variables.
Q1=zeros(5,nsteg);
Q1(:,1)= Q0;

Q2=zeros(5,nsteg);
Q2(:,1)= Q0;

p_acc=zeros(5,nsteg);
p_acc(:,1)=P1;

Hp=zeros(5,nsteg);
Hp(:,1)= Hp_Q0;

%In the beginning the system is a steady state. This means that the time derivative is zero. In order to have this it is necessary to introduce a term which is due to the difference in initial pressure in accumulator and in the pressure delivered by the pump. 
diff_pacc_Hp=Hp_Q0-(p_acc(:,1)/(rho*g));

%Next the solution process of the differential equations are implemented. The outer for loop is counting which frequency is studied while the inner  for loop counts which time step are ongoing. The if criteria is the small perturbation in volume flow which happens when 10% of tmax is calculated. This is done to check that the system is at steady state before the perturbation. The equations are solved with Euler's method.
for j=1:5
    for i=1:nsteg-1
        if i==nsteg/10
            Q2(j,i)=Q2(j,i)*1.001;
        end
        Hp(j,i)=finn_Hp(Q2(j,i), a(j), b(j), c(j),Qstar(j), H0(j));
        Q2(j,i+1)=Q2(j,i)+deltat*3600*g*A/L2*(Hp(j)-1/(rho*g)*p_acc(j,i)-diff_pacc_Hp(j));
        p_acc(j,i+1)=p_acc(j,i)+deltat*(Q2(j,i)-Q1(j,i))/3600*(p0(j)*kappa/(volumeexp(j)*A_acc*L_acc));%+(1/A_acc*rho*g));
        Q1(j,i+1)=Q1(j,i)+deltat*3600*g*A/L1*(p_acc(j,i)/(rho*g)-k(j)*Q1(j,i)^2+diff_pacc_Hp(j)-Hst);
    end
end

%Figure 3 plots the pressure in the accumulator over time. 
figure(3)
plot(t,p_acc)
legend('Surge shaft level, P [Pa]')
xlabel('Time [s]')
ylabel('Surge shaft level, P [Pa]')

%The frequency of the mass oscillations are found by looking at the period of the oscillations. This is done by taking the difference between the highest and the lowest amplitude of the oscillations of the pressure in the accumulator. The frequency is then one divided by the period. 
T=zeros(1,5);
frek=T;
for i=1:5
    [imax(i) jmax(i)]=max(Q2(i,:));
    [imin(i) jmin(i)]=min(Q2(i,:));
    T(i)=2*(t(jmax(i))-t(jmin(i)));
    frek(i)=abs(1/T(i));
end

%The power input, dissipation and power difference is calculated next. The power input is given by the pump characteristic while the dissipation is given by the system characteristic.
p=zeros(5,length(t));
p_in=zeros(5,length(t));
p_dis=zeros(5,length(t));
for i=1:5
    for j=1:nsteg-1
        p_in(i,j)=rho*g*Q2(i,j)*Hp(i,j)/3600;
        p_dis(i,j)=rho*g/3600*(Hst+k(i)*Q1(i,j)^2)*Q1(i,j);
        p(i,j)=p_in(i,j)-p_dis(i,j);
    end
end

%Figure 4 plots the pressure oscillations for the given operational point
figure(4)
plot(t,p)
xlabel('Time [s]')
ylabel('Power [W]')
legend('45 Hz','30 Hz','35 Hz','40 Hz','45 Hz')

%The mass oscillation frequencies are saved in a matrix called Res
Res={'Frequency [Hz]','Helmholtz [Hz]','Mass Oscillations [Hz]';
    25,helmholtz(1),f_torb(1); ...
    30,helmholtz(2),f_torb(2);...
    35,helmholtz(3),f_torb(3);...
    40,helmholtz(4),f_torb(4);...
    45,helmholtz(5),f_torb(5)};

%The Res matrix is here written to Excel. 
xlswrite('Frekvenskalkulasjon.xlsx',Res,2,'A2:C7')

%The frequency of the mass oscillations are written to Excel. The frequency varies with operational point and with rotational frequency.
if Q0<50
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D12:D16')
elseif Q0==50
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D21:D25')
elseif Q0==100
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D31:D35')
elseif Q0==150
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D40:D44')
elseif Q0==200
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D49:D53')
elseif Q0==250
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D58:D62')
elseif Q0==300
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D67:D71')
elseif Q0==350
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D76:D80')
elseif Q0==400
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D85:D89')
elseif Q0==500
    xlswrite('Frekvenskalkulasjon.xlsx',frek',2,'D94:D98')
end

%The posisstable is also written to Excel
if Q0<50
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F12:F16')
elseif Q0==50
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F21:F25')
elseif Q0==100
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F31:F35')
elseif Q0==150
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F40:F44')
elseif Q0==200
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F49:F53')
elseif Q0==250
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F58:F62')
elseif Q0==300
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F67:F71')
elseif Q0==350
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F76:F80')
elseif Q0==400
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F85:F89')
elseif Q0==500
    xlswrite('Frekvenskalkulasjon.xlsx',posisstable,2,'F94:F98')
end

%At the end of the script the code is transformed to latex. 
%dyn=publish('Dynamisk.m','latex');